/*  em2_gpio GPIO functions
12/2/2009 Albert McGilvra
*/
#include "em2_gpio.h"

void Init_GPIO(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;  //define structure for configuring GPIO

  //Enable clocks for GPIOA, GPIOB, GPIOC, GPIOD, GPIOE, and AFIO
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC |
                         RCC_APB2Periph_GPIOD | RCC_APB2Periph_GPIOE | RCC_APB2Periph_AFIO, ENABLE);

  //Reset all GPIO registers to default values
  GPIO_AFIODeInit();
  GPIO_DeInit(GPIOA);
  GPIO_DeInit(GPIOB);
  GPIO_DeInit(GPIOC);
  GPIO_DeInit(GPIOD); 
  GPIO_DeInit(GPIOE); 

  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz; //set port speed
  
  //LED's: PD5 and PD6
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;        
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  //Pwr Ck PB (PB5) and Mute PB (PD7)
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_5;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;  //input with pull-up        
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;  //input with pull-up        
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  //Indicators, active low:
  //SD (PE6), ERR (PE5), DONE (PE4), ALM (PE3), WRN (PE2), Spare1 (PE1), Spare2 (PE0)
  GPIO_InitStructure.GPIO_Pin = 0b0000000001111111;  //configure lower 7-bits
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;        
  GPIO_Init(GPIOE, &GPIO_InitStructure);
  GPIO_SetBits(GPIOE, 0b01111111);  //set high - turn em all off

  //Audio Tone Generator, active high (PD1)
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;        
  GPIO_Init(GPIOD, &GPIO_InitStructure);
  
  //Relay, active low (PB6)
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;        
  GPIO_Init(GPIOB, &GPIO_InitStructure);
  GPIO_SetBits(GPIOB, GPIO_Pin_6);  //turn off relay

  //Digital Inputs DIn1 - DIn4
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;        
  GPIO_Init(GPIOD, &GPIO_InitStructure);

  //GPIO config for peripheral devices is in the respective device.c files. 
}