/**
  ******************************************************************************
  * @file    SDIO/stm32f10x_it.c 
  * @author  MCD Application Team
  * @version V3.1.0
  * @date    06/19/2009
  * @brief   Main Interrupt Service Routines.
  *          This file provides template for all exceptions handler and 
  *          peripherals interrupt service routine.
  ******************************************************************************
  * @copy
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2009 STMicroelectronics</center></h2>
  */ 

/* Includes ------------------------------------------------------------------*/
#include "stm32f10x_it.h"
//!!!IAP #include "sdcard.h"
#include "global.h"
//!!!IAP #include "em2_rtc.h"
#include <stdlib.h>

/** @addtogroup STM32F10x_StdPeriph_Examples
  * @{
  */

/** @addtogroup SDIO_Example
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

//N1 Speed Variables
__IO uint16_t ic4_2_newvalue = 0, ic4_2_oldvalue = 0, first_ic4_2 = 1;
__IO uint16_t TIM4_2_CaptureNumber = 0; //number of captures
__IO uint32_t TIM4_2_Capture = 0;       //clk count since last ISR
__IO uint32_t TIM4_2_Total = 0;         //total clk count
uint8_t N1_FreqReadyFlag;           	//A total count is ready.  Exposed in global.h 
float N1_Freq;                        	//frequency of N1 input on TIM4, ch2.  Exposed in global.h 
uint16_t N1Cap, N1CapOld;               //Latest and saved capture values
uint16_t N1CapTH;                       //Max allowed delta for N1 pulse width, for filtering spurious frequency spikes
uint8_t N1CapFlag;                      //flags that a pulse has been captured

//N2 Speed Variables
__IO uint16_t ic4_3_newvalue = 0,  ic4_3_oldvalue = 0, first_ic4_3 = 1;
__IO uint16_t TIM4_3_CaptureNumber = 0; //number of captures
__IO uint32_t TIM4_3_Capture = 0;       //clk count since last ISR
__IO uint32_t TIM4_3_Total = 0;         //total clk count
uint8_t N2_FreqReadyFlag;           	//A total count is ready.  Exposed in global.h  
float N2_Freq;                        	//frequency of N1 input on TIM4, ch3.  Exposed in global.h 
uint16_t N2Cap, N2CapOld;               //Latest and saved capture values
uint16_t N2CapTH;                       //Max allowed delta for N1 pulse width, for filtering spurious frequency spikes
uint8_t N2CapFlag;                      //flags that a pulse has been captured

//Nr Speed Variables
__IO uint16_t ic4_4_newvalue = 0,  ic4_4_oldvalue = 0, first_ic4_4 = 1;
__IO uint16_t TIM4_4_CaptureNumber = 0; //number of captures
__IO uint32_t TIM4_4_Capture = 0;       //clk count since last ISR
__IO uint32_t TIM4_4_Total = 0;         //total clk count
uint8_t Nr_FreqReadyFlag;           	//A total count is ready.  Exposed in global.h  
float Nr_Freq;                        	//frequency of N1 input on TIM4, ch3.  Exposed in global.h 
uint16_t NrCap, NrCapOld;               //Latest and saved capture values
uint16_t NrCapTH;                       //Max allowed delta for N1 pulse width, for filtering spurious frequency spikes
uint8_t NrCapFlag;                      //flags that a pulse has been captured



uint32_t SysTickCnt;                    //incremented every ms
uint8_t LoopTimerExpired, DelayTimerExpired;  //set by sys tick isr
uint32_t msDelay; //decremented by sys tick isr


/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M3 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief  This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {}
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {}
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {}
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {}
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval None
  */
void DebugMon_Handler(void)
{
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles PendSV_Handler exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTickHandler(void) //Set for 1mS interrupt period main.c file of EM2 project
//Name of this routine should be should be "SysTick_Handler" per STM lib v3, but Rowley's system files use the old name w/no underscore
{
//!!!IAP
//  SysTickCnt++;  //increment count
// 
//  if (SysTickCnt % 10 == 0)  //every 10ms
//  {
//    LoopTimerExpired = 1; //set loop timer flag
//    HClk++;  //global hundreths clock, used for software timers.
//  }
//
//  if (AutoFeedWDTmr)  //if we are set to auto feed the timer, i.e., during User Settings
//    if (SysTickCnt % 100 == 0)  //every 100ms
//      IWDG_ReloadCounter(); //feed wd timer   
//  
//  if (msDelay == 0)
//    DelayTimerExpired = 1;  //set ms delay timer flag
//  else
//    msDelay--;              //count down
}

/******************************************************************************/
/*            STM32F10x Peripherals Interrupt Handlers                        */
/******************************************************************************/

/**
  * @brief  This function handles SDIO global interrupt request.
  * @param  None
  * @retval None
  */
void SDIO_IRQHandler(void)
{
  /* Process All SDIO Interrupt Sources */
//!!!IAP   SD_ProcessIRQSrc();
}
/******************************************************************************/
void TIM4_IRQHandler(void)  //for EM2 N1, N2, and Nr speeds.  
/* The Nx inputs are pre-divided by 8 then go to cinput capture lines.
This interrupt fires whenever a rising edge occurs on one of the capture lines (every 8 speed pulses) or counter wraps
See em2_tim.c for more explanation.
*/
{

  if(TIM_GetITStatus(TIM4, TIM_IT_Update) == SET) //if counter wrapped
  {
    TIM_ClearITPendingBit(TIM4, TIM_IT_Update);  //Clear TIM4 update interrupt pending bit
    first_ic4_2 = 1;  //indicate first input capture
    first_ic4_3 = 1;
    first_ic4_4 = 1;

   //N1 frequncy calculation
    if (TIM4_2_CaptureNumber > 0 && TIM4_2_Total <= 65535)  //if at least one capture has taken place since last wrap
    {
      N1_Freq = TIM4_2_CaptureNumber * 800000.0 / TIM4_2_Total;  //input frequency by divided by 8
      N1_FreqReadyFlag = 1;
      TIM4_2_Total = 0;
      TIM4_2_CaptureNumber = 0;
      N1CapFlag = 0;  
    }
    else
    {
      N1_Freq = 0;
      N1_FreqReadyFlag = 1;  //yes, even 0 is a valid freq measurement
      TIM4_2_Total = 0;
      TIM4_2_CaptureNumber = 0; //5/11/2012 NOTE TO SELF: did not have this line here before, may be why we had double size spikes.
      N1CapFlag = 0;  
    }

    //N2 frequncy calculation
    if (TIM4_3_CaptureNumber > 0 && TIM4_3_Total <= 65535)  //if at east one capture has taken place since last wrap
    {
      N2_Freq = TIM4_3_CaptureNumber * 800000.0 / TIM4_3_Total;  //input frequency by divided by 8
      N2_FreqReadyFlag = 1;
      TIM4_3_Total = 0;
      TIM4_3_CaptureNumber = 0;
      N2CapFlag = 0;  
    }
    else
    {
      N2_Freq = 0;
      N2_FreqReadyFlag = 1;  //yes, even 0 is a valid freq measurement
      TIM4_3_Total = 0;
      TIM4_3_CaptureNumber = 0; //5/11/2012 NOTE TO SELF: did not have this line here before, may be why we had double size spikes.
      N2CapFlag = 0;  
    }

    //Nr frequncy calculation
    if (TIM4_4_CaptureNumber > 0 && TIM4_4_Total <= 65535)  //if at east one capture has taken place since last wrap
    {
      Nr_Freq = TIM4_4_CaptureNumber * 800000.0 / TIM4_4_Total;  //input frequency by divided by 8
      Nr_FreqReadyFlag = 1;
      TIM4_4_Total = 0;
      TIM4_4_CaptureNumber = 0;
      NrCapFlag = 0;  
    }
    else
    {
      Nr_Freq = 0;
      Nr_FreqReadyFlag = 1;  //yes, even 0 is a valid freq measurement
      TIM4_4_Total = 0;
      TIM4_4_CaptureNumber = 0; //5/11/2012 NOTE TO SELF: did not have this line here before, may be why we had double size spikes.
      NrCapFlag = 0;  
    }
    
    return;  //on the outside chance the counter wrapped at the same time as an input capture, returning prevents indeterminate behavior
  }//end Update (end If Counter Wrapped)
  
  //N1 Speed, connected to TIM4, Channel 2
  if(TIM_GetITStatus(TIM4, TIM_IT_CC2) == SET) //if Ch2 caused interrupt
  {
    TIM_ClearITPendingBit(TIM4, TIM_IT_CC2);  //Clear TIM4 Capture compare interrupt pending bit
    if (first_ic4_2)
    {  
      first_ic4_2 = 0;  //clear first flag
      ic4_2_oldvalue = TIM_GetCapture2(TIM4);    //save the Input Capture value
    }
    else
    {
      ic4_2_newvalue = TIM_GetCapture2(TIM4);  //get current counter value
      TIM4_2_Capture = ic4_2_newvalue - ic4_2_oldvalue; //calc delta
      ic4_2_oldvalue = ic4_2_newvalue;  //update old value
      N1CapFlag = 1;  //indicates at least one capture, valid or otherwise

      if (TIM4_2_Capture <= 65535) //if captured pulse width is valid 
      {
        N1CapOld = N1Cap; 
        N1Cap = TIM4_2_Capture;
        if (abs(N1Cap - N1CapOld) <= N1CapTH)
        {
          TIM4_2_Total += TIM4_2_Capture;   //calc total
          TIM4_2_CaptureNumber++;           //increment capture number
        }
      }
    }
  }//end N1

  //N2 Speed, connected to TIM4, Channel 3
  if(TIM_GetITStatus(TIM4, TIM_IT_CC3) == SET) //if Ch3 caused interrupt
  {
    TIM_ClearITPendingBit(TIM4, TIM_IT_CC3);  //Clear TIM4 Capture compare interrupt pending bit
    if (first_ic4_3)
    {  
      first_ic4_3 = 0;  //clear first flag
      ic4_3_oldvalue = TIM_GetCapture3(TIM4);    //save the Input Capture value
    }
    else
    {
      ic4_3_newvalue = TIM_GetCapture3(TIM4);  //get current counter value
      TIM4_3_Capture = ic4_3_newvalue - ic4_3_oldvalue; //calc delta
      ic4_3_oldvalue = ic4_3_newvalue;  //update old value
      N2CapFlag = 1;  //indicates at least one capture, valid or otherwise

      if (TIM4_3_Capture <= 65535) //if captured pulse width is valid 
      {
        N2CapOld = N2Cap; 
        N2Cap = TIM4_3_Capture;
        if (abs(N2Cap - N2CapOld) <= N2CapTH)
        {      
          TIM4_3_Total += TIM4_3_Capture;   //calc total
          TIM4_3_CaptureNumber++;           //increment capture number
        }
      }
    }
  }//end N2
      
  //Nr Speed, connected to TIM4, Channel 4 //if Ch4 caused interrupt
  if(TIM_GetITStatus(TIM4, TIM_IT_CC4) == SET) 
  {
    TIM_ClearITPendingBit(TIM4, TIM_IT_CC4);  //Clear TIM4 Capture compare interrupt pending bit
    if (first_ic4_4)
    {  
      first_ic4_4 = 0;  //clear first flag
      ic4_4_oldvalue = TIM_GetCapture4(TIM4);    //save the Input Capture value
    }
    else
    {
      ic4_4_newvalue = TIM_GetCapture4(TIM4);  //get current counter value
      TIM4_4_Capture = ic4_4_newvalue - ic4_4_oldvalue; //calc delta
      ic4_4_oldvalue = ic4_4_newvalue;  //update old value
      NrCapFlag = 1;  //indicates at least one capture, valid or otherwise

      if (TIM4_4_Capture <= 65535) //if captured pulse width is valid 
      {
        NrCapOld = NrCap; 
        NrCap = TIM4_4_Capture;
        if (abs(NrCap - NrCapOld) <= NrCapTH)
        {      
          TIM4_4_Total += TIM4_4_Capture;   //calc total
          TIM4_4_CaptureNumber++;           //increment capture number
        }
      }
    }
  }//end Nr        
}//end TIM4_IRQ
/******************************************************************************/
void RTC_IRQHandler(void)
{
//!!!IAP
//  uint8_t leap;
//  uint32_t value;
//  uint32_t count;
//
//  if (RTC_GetITStatus(RTC_IT_SEC) != RESET)
//  {
//     
//    RTC_ClearITPendingBit(RTC_IT_SEC);  // Clear the RTC Second interrupt
//    RTC_WaitForLastTask();
//
//    count = RTC_GetCounter();
//    RTC_WaitForLastTask();
//
//
//
//    // Reset RTC Counter when Time is 23:59:59 ------------- Rollover at 24 hours
//    if (count == 0x00015180) // Total seconds in hex
//    {
//      RTC_SetCounter(0x1);
//      RTC_WaitForLastTask();
//      clock.hour = 0;
//      clock.minute = 0;
//      clock.second = 0;     
//
//      // Add a Day
//      clock.day++;
//
//      // Leap year
//      if( ((clock.year%4 == 0) && (clock.year%100 != 0)) || (clock.year%400==0) ) leap=1;  
//      else leap=0;
//
//      // Rollover Month and day
//      switch(clock.month)
//      {
//        case 0:  break;
//        case 1:  if(clock.day > 31) {value = clock.day/31;clock.day = clock.day-(31*value);clock.month = clock.month+value;}break;
//        case 2:  if(clock.day > (28+leap)) {value = clock.day/(28+leap);clock.day = clock.day-((28+leap)*value);clock.month = clock.month+value;}break;
//        case 3:  if(clock.day > 31) {value = clock.day/31;clock.day = clock.day-(31*value);clock.month = clock.month+value;}break;
//        case 4:  if(clock.day > 30) {value = clock.day/30;clock.day = clock.day-(30*value);clock.month = clock.month+value;}break;
//        case 5:  if(clock.day > 31) {value = clock.day/31;clock.day = clock.day-(31*value);clock.month = clock.month+value;}break;
//        case 6:  if(clock.day > 30) {value = clock.day/30;clock.day = clock.day-(30*value);clock.month = clock.month+value;}break;
//        case 7:  if(clock.day > 31) {value = clock.day/31;clock.day = clock.day-(31*value);clock.month = clock.month+value;}break;
//        case 8:  if(clock.day > 31) {value = clock.day/31;clock.day = clock.day-(31*value);clock.month = clock.month+value;}break;
//        case 9:  if(clock.day > 30) {value = clock.day/30;clock.day = clock.day-(30*value);clock.month = clock.month+value;}break;
//        case 10: if(clock.day > 31) {value = clock.day/31;clock.day = clock.day-(31*value);clock.month = clock.month+value;}break;
//        case 11: if(clock.day > 30) {value = clock.day/30;clock.day = clock.day-(30*value);clock.month = clock.month+value;}break;
//        case 12: if(clock.day > 31) {value = clock.day/31;clock.day = clock.day-(31*value);clock.month = clock.month+value;}break;
//      }
//
//      if(clock.month > 12) 
//      {
//        value = clock.month/12;
//        clock.month = clock.month-(12*value);
//        clock.year = clock.year+value;
//      }
//      BKP_WriteBackupRegister(BKP_DR2,clock.month);
//      BKP_WriteBackupRegister(BKP_DR3,clock.day); 
//      BKP_WriteBackupRegister(BKP_DR1,clock.year);
//
//
//    }
//    else
//    {
//      clock.hour = count/3600;
//      clock.minute = (count%3600)/60;
//      clock.second = (count%3600)%60;
//    }
//
//
//  }
}
/******************************************************************************/
/******************************************************************************/
/*                 STM32F10x Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f10x_xx.s).                                            */
/******************************************************************************/

/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
/*void PPP_IRQHandler(void)
{
}*/

/**
  * @}
  */ 

/**
  * @}
  */ 

/******************* (C) COPYRIGHT 2009 STMicroelectronics *****END OF FILE****/
