#include "stm32f10x.h"  //ST CMSIS Cortex-M3 Device Peripheral Access Layer Header File. 
#include "em2_gpio.h"

//Global variabes, also declare in global.h as "extern"


//Private functions
void RCC_Configuration(void);  //Configures the different system clocks.
void NVIC_Configuration(void); //Configures SDIO IRQ channel

/*****************************************************************************/
int main()  //returns 0 if all ok, negative return codes indicate an error.
{
  uint32_t loopcnt = 0; //local variables are not initialized by default.  Global varaibles are initialized to zero by default.
  uint32_t cyclecnt = 0;  
//  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x8000); //relocate the vector table to same as loader address: 
                                                   //Properties - Target Loader Options - Loader Parameter
  
  RCC_Configuration();  //Configures the different system clocks.
  NVIC_Configuration(); //Configures SDIO IRQ channel
  Init_GPIO();          //Init all EM2 GPIO
  GPIO_SetBits(GPIOD, GPIO_Pin_5);  //turn off green led 

  while(1)
  {
    loopcnt++;
    if (loopcnt == 1000000)
    {
      GPIO_ResetBits(GPIOD, GPIO_Pin_6);  //turn on red led
    }

    if (loopcnt == 2000000)
    {
      GPIO_SetBits(GPIOD, GPIO_Pin_6);  //turn off red led
      loopcnt = 0;
      cyclecnt++;
    }
    if (cyclecnt == 5)
      break;  //exit while, jump to IAP

    
    //GPIO_ResetBits(GPIOD, GPIO_Pin_5);  //turn on green led  
    //GPIO_SetBits(GPIOD, GPIO_Pin_5);  //turn off green led 
    //!!todo feed wd tmr
 }//end while(1)

  typedef  void (*pFunction)(void);
  #define IAP_ADDR 0x806E000  //address of in-application programmer bootloader
  pFunction Jump_To_IAP;
  uint32_t JumpAddress;
  
  /* Jump to bootloader */
  JumpAddress = IAP_ADDR + 0x235;      
  Jump_To_IAP = (pFunction)JumpAddress;  
  __set_MSP(*(__IO uint32_t*) IAP_ADDR);  //Initialize stack Pointer
  Jump_To_IAP(); //go to IAP now

}//end main

/*****************************************************************************/
void RCC_Configuration(void)  //Configures the different system clocks.
{
  /* Setup the microcontroller system. Initialize the Embedded Flash Interface,
     initialize the PLL and update the SystemFrequency variable. */
  SystemInit();
}
/*****************************************************************************/
void NVIC_Configuration(void) //Configures SDIO IRQ channel
{
  NVIC_InitTypeDef NVIC_InitStructure;

  /* Configure the NVIC Preemption Priority Bits */
  // Priority Group           Priority       SubPriority    
  //--------------------------------------------------------
  // NVIC_PriorityGroup_0     0-bits (0)     4-bits (0-15)
  // NVIC_PriorityGroup_1     1-bits (0-1)   3-bits (0-7)
  // NVIC_PriorityGroup_2     2-bits (0-3)   2-bits (0-3)
  // NVIC_PriorityGroup_4     3-bits (0-7)   1-bits (0-1)
  // NVIC_PriorityGroup_4     4-bits (0-15)  0-bits (0)
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);

  //SDIO Interrupt
  NVIC_InitStructure.NVIC_IRQChannel = SDIO_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1; //V55.11, was = 0
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  //RTC Interrupt 
  NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2; //v55.11, was = 1
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

}
/*****************************************************************************/
#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {}
}
#endif
/*****************************************************************************/
//end of file